-- GRAPHICS LUA SHELL FOR TTS SOFTWARE "DEMAGOG"
-- (It must be in ANSI encoding)
-- (c) Benedict Lee


-- FLOAT CANVAS ----------------------------------------------------------
-- X-axis pointed RIGHT
-- Y-axis pointing UP
-- Coordinates are real numbers, either positive or negative 
-- 0, 0 - is the origin
-- Units of measure can be ANY
-- The conversion to pixel coordinates of the screen is done automatically 
--------------------------------------------------------------------------


GROSS = 1.797693E+308


-- Convert hex-string color code to number color code
function HexColorToNum(h)
  if type(h) ~= 'string' or h == '' then return -1 end  
  local s = string.sub(h,5,6)..string.sub(h,3,4)..string.sub(h,1,2)
  return tonumber(s,16)
end


-- Set a new name to graphics window ('Infographics' by default)
function Headline(name)
  if name == nil then name = '_NO_NAME_' end
  Grafica(0,name)
end


-- Set the bounds of work area
-- x1,y1 - left bottom corner
-- x2, y2 - right top corner
function Area(x1,y1,x2,y2)
  Grafica(1,x1,y1,x2,y2)
end


-- Set pen parameters
function Pen(h,width)
  Grafica(2,HexColorToNum(h),width)
end


-- Move pen to x,y-point
function MoveTo(x,y)
  Grafica(3,x,y)
end


-- Draw line from current point to x,y-point
function LineTo(x,y)
  Grafica(4,x,y)
end


function RandColor()
  local n
  local c = '123456789ABCDEF0'
  local h = ''
  for i = 1,6 do
    n = math.random(1,16)
    h = h..string.sub(c,n,n)
  end
  return h
end


function RandColorLight()
  local n
  local c = '89ABCDEF'
  local h = ''
  for i = 1,6 do
    n = math.random(1,8)
    h = h..string.sub(c,n,n)
  end
  return h
end


function RandColorDark()
  local n
  local c = '01234567'
  local h = ''
  for i = 1,6 do
    n = math.random(1,8)
    h = h..string.sub(c,n,n)
  end
  return h
end


-- Set brush fill-color
-- If h = '' then figures will not filled!
math.randomseed(os.clock()*1000000000)
function Brush(h)
  if h == nil then h = RandColor() end
  Grafica(5,HexColorToNum(h))
end


-- Set x,y-point color
function Pix(x,y,h)
  Grafica(6,x,y,HexColorToNum(h))
end


-- Set font parameters
function Font(name,size,h)
  Grafica(7,name,size,HexColorToNum(h))
end


-- Draw text
-- mode = 0 - ajust Left (by default)
-- mode = 1 - Right
-- mode = 2 - Center
-- useful constants:
taLeft, taRight, taCenter = 0, 1, 2
function TextOut(x,y,text,mode)
  if mode == nil then mode = 0 end
  Grafica(8,x,y,text,mode)
end


-- x1,y1 - left bottom corner of rectangle around ellipse
-- x2, y2 - right top corner of rectangle around ellipse
function Ellipse(x1,y1,x2,y2)
  Grafica(9,x1,y1,x2,y2)
end


-- x1,y1 - left bottom corner
-- x2, y2 - right top corner
function Rectangle(x1,y1,x2,y2)
  Grafica(10,x1,y1,x2,y2)
end


-- x1,y1 - left bottom corner
-- x2, y2 - right top corner
function RoundRect(x1,y1,x2,y2)
  Grafica(11,x1,y1,x2,y2)
end


-- p is array {x1,y1,x2,y2, ... ,xN,yN}
-- must by N <= 1000
function Polygon(p)
  local c = table.concat(p,'\3')
  Grafica(12,c)
end


function Dot(x,y,d)
  Grafica(13,x,y,d)
end


-- Create sector of ellipse/cyrcle
-- x1,y1 - left bottom corner of rectangle around ellipse
-- x2, y2 - right top corner of rectangle around ellipse
-- t1 - start angle (radian)
-- t2 - end angle (radian)
function Pie(x1,y1,x2,y2,t1,t2)
  Grafica(14,x1,y1,x2,y2,t1,t2)
end


-- Fills a closed area with the color of the current brush from the point x, y
-- if mode = true, then all adjacent areas are filled, the color of which is equal to h
-- if mode = false, then all adjacent areas are filled, the color of which is NOT equal to h
function Fill(x, y, h, mode)
  if mode == nil then mode = true end
  Grafica(15,x,y,HexColorToNum(h),mode)
end


-- Coordinates x, y of the point on an ellipse with center x, y and semi-axes a and b
-- t is the angle (in radians) between the X axis and the vector from the center to this point 
-- (so you can translate the polar coordinates in Cartesian)
function Polar(x,y,a,b,t)
  return x+a*math.cos(t), y+b*math.sin(t) 
end

-- ... --

function ColorDialog()
  return Grafica(249)
end


function SaveFrame(f)
  Grafica(250,f)
end


-- Pause (milliseconds)
function Delay(msec)
  Grafica(251,msec)
end


-- Clear canvas
function Clear()
  Grafica(252)
end


-- Hide the graphics window
function Hide()
  Grafica(253)
end


-- Show the graphics window
function Show()
  Grafica(254)
end


-- Create image in graphics window
function Render()
  Grafica(255)
end


-- Draw pie chart
-- c - caption
-- g = {{Name1,percent1,color1}, ... ,{NameN,percentN,colorN}}
-- N <= 16
function PieChart(c,g)
  local t1, t2, x, y
  Brush('')
  Pen(Silver,1)
  Rectangle(0,175,2100,1400)
  Font('Arial',40,Black)
  TextOut(0,100,c)
  t1 = 0
  Font('Arial',32,Black)
  for i = 1,16 do  -- not more then 16 objects!
    if g[i] ~= nil then
      t2 = t1+g[i][2]*math.pi/50
      Brush(g[i][3])
      Pie(125,295,1125,1295,t1,t2)
      x = 1350
      y = 1375-65*i
      TextOut(x,y,round(g[i][2],1))
      Brush('')
      TextOut(x+150,y,g[i][1])
      t1 = t2
    end
  end
end


-- Draw horizontal bar chart
function BarChart(c,g)
    local x, y, xscale
    local W = 2100
    bl, bb, br, bt = 0, 1400,  W, 1400
    Brush('')
    Pen(Silver,1)
    Font('Arial',32,Black)
    local xmax = -GROSS
    for i = 1, #g do
        if g[i][2] > xmax then xmax = g[i][2] end    
    end
  for i = 1, #g do  
    if g[i] ~= nil then
      Brush(g[i][3])
      x = 500 --1325
      y = 1400-76*i
     -- Rectangle(x+50,y-56,x+0.65*W*g[i][2]/xmax+50,y+4)
      Rectangle(x+50,y-58,x+0.65*W*g[i][2]/xmax+50,y+6)
      if g[i][2] >= 1.5 then
         TextOut(x+66,y,round(g[i][2],1))
      else
           Brush('')
          TextOut(x+150,y,round(g[i][2],1))
      end 
      Brush('')
      TextOut(x,y,g[i][1],taRight)
    end
    bb = bb - 85
  end
    Font('Arial',40,Black)
    TextOut(bl,bb-40,c)
    Brush('')
    Rectangle( bl, bb+50, br, bt )
end


-- Draw a graph of the data set g
-- c - caption under graph
-- g - data table: {{x1,y1}, ... {xN,yN}}
-- v -parameter table with keys: 
-- AspectRatio, SinglePoints, Framing, penColor, penWidth, xStep, yStep, xEvery, yEvery, xmin, ymin, xmax, ymax
function Graph(c,g,v)
    local W, H -- the size of the working area on which to scale the data
    if v.AspectRatio then
        W = math.ceil(math.sqrt(2^22*v.AspectRatio))
        H = math.ceil(math.sqrt(2^22/v.AspectRatio))
    else
        W = 2048
        H = 2048
    end
    if not v.xEvery then v.xEvery = 1 end
    if not v.yEvery then v.yEvery = 1 end
    local xscale, yscale
    -- minimum graph area size for visibility of all data
    local xmin, xmax = GROSS, -GROSS
    local ymin, ymax = GROSS, -GROSS
    for i = 1, #g do
        if g[i][1] < xmin then xmin = g[i][1] end
        if g[i][1] > xmax then xmax = g[i][1] end    
        if g[i][2] < ymin then ymin = g[i][2] end
        if g[i][2] > ymax then ymax = g[i][2] end
    end
    -- user can extend the graph area size up to more comfortable
    if v.xmin then xmin = math.min(xmin, v.xmin) end
    if v.xmax then xmax = math.max(xmax, v.xmax) end
    if v.ymin then ymin = math.min(ymin, v.ymin) end
    if v.ymax then ymax = math.max(ymax, v.ymax) end
    -- calculate the scale factors for the axes
    xscale = (xmax-xmin)/W
    yscale = (ymax-ymin)/H
    local xmin_, xmax_ = xmin/xscale, xmax/xscale
    local ymin_, ymax_ = ymin/yscale, ymax/yscale
    Brush('')
    -- draw graph
    if not v.SinglePoints then
        Pen(v.penColor,v.penWidth)
        MoveTo(g[1][1]/xscale,g[1][2]/yscale)
        for i = 2, #g do
            LineTo(g[i-1][1]/xscale,g[i-1][2]/yscale)
        end
    else
        Brush(v.penColor)
        Pen(Black)
        d = v.penWidth
        for i = 1, #g do
            Dot(g[i][1]/xscale,g[i][2]/yscale,d)
        end
    end
    --
    Brush('')
    Pen(Silver,1)
    Font('Arial',32,Black)
    -- draw a coordinate grid, if its parameters are given a step along the axes
    if v.xStep ~= nil and (xmax-xmin) < 100*v.xStep then
        local xstart = (math.floor(xmin/v.xStep) + 1)*v.xStep
        while xstart < xmax do
           MoveTo(xstart/xscale,ymin_)
           LineTo(xstart/xscale,ymax_)      
           if xstart % v.xEvery == 0 then TextOut(xstart/xscale,ymin_-50,xstart) end
           xstart = xstart+v.xStep
        end
    end
    if v.yStep ~= nil and (ymax-ymin) < 100*v.yStep then
        local ystart = (math.floor(ymin/v.yStep) + 1)*v.yStep
        while ystart < ymax do
            MoveTo(xmin_,ystart/yscale)
            LineTo(xmax_,ystart/yscale)      
            if ystart % v.yEvery == 0 then TextOut(xmin_-150,ystart/yscale,ystart) end
            ystart = ystart+v.yStep
        end
    end
    -- framing work area
    -- and print the extreme values on the axis
    if v.Framing then
        Rectangle(xmin_,ymin_,xmax_,ymax_)  
        TextOut(xmin_-150,ymin_,ymin)
        TextOut(xmin_-150,ymax_+50,ymax)
        TextOut(xmin_,ymin_-50,xmin)
        TextOut(xmax_,ymin_-50,xmax)
        Font('Arial',42,Black)
        if c then 
            local d = string.split(c,'\r')
            for i = 1,#d do
                TextOut(xmin_,ymin_-(90+60*i),d[i])
            end
        end
    end
end


-- Draw a histogram of the data set g
-- c - caption under graph
-- g - data table: {x1, ... xN}
-- v -parameter table with keys: 
-- AspectRatio, Framing, fillColor, xStep, yStep, xEvery, yEvery, xmin, ymin, xmax, ymax
function Histogram(c,g,v)
    local W, H -- the size of the working area on which to scale the data
    if v.AspectRatio then
        W = math.ceil(math.sqrt(2^22*v.AspectRatio))
        H = math.ceil(math.sqrt(2^22/v.AspectRatio))
    else
        W = 2048
        H = 2048
    end
    if not v.xEvery then v.xEvery = 1 end
    if not v.yEvery then v.yEvery = 1 end
    local xscale, yscale
    -- create histogram table and calculate
    -- minimum graph area size for visibility of all histogram
    h = {}
    local ymin, ymax = GROSS, -GROSS  --0, 0
    local jmin, jmax = 2^32-1, -2^32-1  --0, 0
    if not v.xStep then goto HALT end
    for i = 1, #g do
        j = math.floor(g[i]/v.xStep)
        if not h[j] then 
            h[j] = {j*v.xStep, 0}
        else
            h[j][2] = h[j][2]+1
        end 
        if j < jmin then jmin = j end
        if j > jmax then jmax = j end 
        if h[j][2] > ymax then ymax = h[j][2] end    
    end  
    local xmin, xmax = (jmin)*v.xStep, (jmax+1)*v.xStep
    -- user can extend the graph area size up to more comfortable
    if v.xmin then xmin = math.min(xmin, v.xmin) end
    if v.xmax then xmax = math.max(xmax, v.xmax) end
    if v.ymin then ymin = math.min(ymin, v.ymin) end
    if v.ymax then ymax = math.max(ymax, v.ymax) end
    -- calculate the scale factors for the axes
    xscale = (xmax-xmin)/W
    yscale = (ymax-ymin)/H
    local xmin_, xmax_ = xmin/xscale, xmax/xscale
    local ymin_, ymax_ = ymin/yscale, ymax/yscale
    Brush(v.fillColor)
    -- draw histogram
    for j = jmin,jmax do
        if h[j] then Rectangle((j)*v.xStep/xscale,0,(j+1)*v.xStep/xscale,h[j][2]/yscale) end
    end
    -- 
    Brush('')
    Pen(Silver,1)
    Font('Arial',32,Black)
    -- draw a coordinate grid, if its parameters are given a step along the axes
    if v.xStep ~= nil and (xmax-xmin) < 100*v.xStep then
        local xstart = (math.floor(xmin/v.xStep) + 1)*v.xStep
        while xstart < xmax do
           MoveTo(xstart/xscale,ymin_)
           LineTo(xstart/xscale,ymax_)      
           if xstart % v.xEvery == 0 then TextOut(xstart/xscale,ymin_-50,xstart) end
           xstart = xstart+v.xStep
        end
    end
    if v.yStep ~= nil and (ymax-ymin) < 100*v.yStep then
        local ystart = (math.floor(ymin/v.yStep) + 1)*v.yStep
        while ystart <= ymax do
            MoveTo(xmin_,ystart/yscale)
            LineTo(xmax_,ystart/yscale)      
            if ystart % v.yEvery == 0 then TextOut(xmin_-150,ystart/yscale,ystart) end
            ystart = ystart+v.yStep
        end
    end
    -- framing work area
    -- if grid step is not set, do not drawing it, but print the extreme values on the axis
    if v.Framing then
        Rectangle(xmin_,ymin_,xmax_,ymax_)  
        TextOut(xmin_-150,ymin_,ymin)
        if not v.yStep then 
            TextOut(xmin_-150,ymax_,ymax)
        end 
        TextOut(xmin_,ymin_-50,xmin)
        if not v.xStep then 
            TextOut(xmax_,ymin_-50,xmax)
        end
        Font('Arial',42,Black)
        TextOut(xmin_,ymin_-150,c)
    end
    ::HALT::
end


-- HTML Color Table
--
--Red tones
--
LightCoral	 = 'F08080'
Salmon		 = 'FA8072'
DarkSalmon	 = 'E9967A'
LightSalmon	 = 'FFA07A'
Crimson		 = 'DC143C'
Red		 = 'FF0000'
FireBrick	 = 'B22222'
DarkRed		 = '8B0000'
--
-- Pink tones
--
Pink		 = 'FFC0CB'
LightPink	 = 'FFB6C1'
HotPink		 = 'FF69B4'
DeepPink	 = 'FF1493'
MediumVioletRed	 = 'C71585'
PaleVioletRed	 = 'DB7093'
--
-- Orange tones
--
LightSalmon	 = 'FFA07A'
Coral		 = 'FF7F50'
Tomato		 = 'FF6347'
OrangeRed	 = 'FF4500'
DarkOrange	 = 'FF8C00'
Orange		 = 'FFA500'
--
-- Yellow tones
--
Gold		 = 'FFD700'
Yellow		 = 'FFFF00'
LightYellow	 = 'FFFFE0'
LemonChiffon	 = 'FFFACD'
LightGoldenrodYellow = 'FAFAD2'
PapayaWhip	 = 'FFEFD5'
Moccasin	 = 'FFE4B5'
PeachPuff	 = 'FFDAB9'
PaleGoldenrod	 = 'EEE8AA'
Khaki		 = 'F0E68C'
DarkKhaki	 = 'BDB76B'
--
-- Violet tones
--
Lavender	 = 'E6E6FA'
Thistle		 = 'D8BFD8'
Plum		 = 'DDA0DD'
Violet		 = 'EE82EE'
Orchid		 = 'DA70D6'
Fuchsia		 = 'FF00FF'
Magenta		 = 'FF00FF'
MediumOrchid	 = 'BA55D3'
MediumPurple	 = '9370DB'
BlueViolet	 = '8A2BE2'
DarkViolet	 = '9400D3'
DarkOrchid	 = '9932CC'
DarkMagenta	 = '8B008B'
Purple		 = '800080'
Indigo		 = '4B0082'
SlateBlue	 = '6A5ACD'
DarkSlateBlue	 = '483D8B'
--
-- Brown tones
--
Cornsilk	 = 'FFF8DC'
BlanchedAlmond	 = 'FFEBCD'
Bisque		 = 'FFE4C4'
NavajoWhite	 = 'FFDEAD'
Wheat		 = 'F5DEB3'
BurlyWood	 = 'DEB887'
Tan		 = 'D2B48C'
RosyBrown	 = 'BC8F8F'
SandyBrown	 = 'F4A460'
Goldenrod	 = 'DAA520'
DarkGoldenRod	 = 'B8860B'
Peru		 = 'CD853F'
Chocolate	 = 'D2691E'
SaddleBrown	 = '8B4513'
Sienna		 = 'A0522D'
Brown		 = 'A52A2A'
Maroon		 = '800000'
--
-- Green tones
--
GreenYellow	 = 'ADFF2F'
Chartreuse	 = '7FFF00'
LawnGreen	 = '7CFC00'
Lime		 = '00FF00'
LimeGreen	 = '32CD32'
PaleGreen	 = '98FB98'
LightGreen	 = '90EE90'
MediumSpringGreen = '00FA9A'
SpringGreen	 = '00FF7F'
MediumSeaGreen	 = '3CB371'
SeaGreen	 = '2E8B57'
ForestGreen	 = '228B22'
Green		 = '008000'
DarkGreen	 = '006400'
YellowGreen	 = '9ACD32'
OliveDrab	 = '6B8E23'
Olive		 = '808000'
DarkOliveGreen	 = '556B2F'
MediumAquamarine = '66CDAA'
DarkSeaGreen	 = '8FBC8F'
LightSeaGreen	 = '20B2AA'
DarkCyan	 = '008B8B'
Teal		 = '008080'
--
-- Blue tones
--
Aqua		 = '00FFFF'
Cyan		 = '00FFFF'
LightCyan	 = 'E0FFFF'
PaleTurquoise	 = 'AFEEEE'
Aquamarine	 = '7FFFD4'
Turquoise	 = '40E0D0'
MediumTurquoise	 = '48D1CC'
DarkTurquoise	 = '00CED1'
CadetBlue	 = '5F9EA0'
SteelBlue	 = '4682B4'
LightSteelBlue	 = 'B0C4DE'
PowderBlue	 = 'B0E0E6'
LightBlue	 = 'ADD8E6'
SkyBlue		 = '87CEEB'
LightSkyBlue	 = '87CEFA'
DeepSkyBlue	 = '00BFFF'
DodgerBlue	 = '1E90FF'
CornflowerBlue	 = '6495ED'
MediumSlateBlue	 = '7B68EE'
RoyalBlue	 = '4169E1'
Blue		 = '0000FF'
MediumBlue	 = '0000CD'
DarkBlue	 = '00008B'
Navy		 = '000080'
MidnightBlue	 = '191970'
--
-- White tones
--
White		 = 'FFFFFF'
Snow		 = 'FFFAFA'
Honeydew	 = 'F0FFF0'
MintCream	 = 'F5FFFA'
Azure		 = 'F0FFFF'
AliceBlue	 = 'F0F8FF'
GhostWhite	 = 'F8F8FF'
WhiteSmoke	 = 'F5F5F5'
Seashell	 = 'FFF5EE'
Beige		 = 'F5F5DC'
OldLace		 = 'FDF5E6'
FloralWhite	 = 'FFFAF0'
Ivory		 = 'FFFFF0'
AntiqueWhite	 = 'FAEBD7'
Linen		 = 'FAF0E6'
LavenderBlush	 = 'FFF0F5'
MistyRose	 = 'FFE4E1'
--
-- Gray tones
--
Gainsboro	 = 'DCDCDC'
LightGrey	 = 'D3D3D3'
LightGray	 = 'D3D3D3'
Silver		 = 'C0C0C0'
DarkGray	 = 'A9A9A9'
DarkGrey	 = 'A9A9A9'
Gray		 = '808080'
Grey		 = '808080'
DimGray		 = '696969'
DimGrey		 = '696969'
LightSlateGray	 = '778899'
LightSlateGrey	 = '778899'
SlateGray	 = '708090'
SlateGrey	 = '708090'
DarkSlateGray	 = '2F4F4F'
DarkSlateGrey	 = '2F4F4F'
Black		 = '000000'
--
-- Base colors
--
Black		 = '000000'
Gray		 = '808080'
Silver		 = 'C0C0C0'
White		 = 'FFFFFF'
Fuchsia		 = 'FF00FF'
Purple		 = '800080'
Red		 = 'FF0000'
Maroon		 = '800000'
Yellow		 = 'FFFF00'
Olive		 = '808000'
Lime		 = '00FF00'
Green		 = '008000'
Aqua		 = '00FFFF'
Teal		 = '008080'
Blue		 = '0000FF'
Navy		 = '000080'
--
-- End of HTML Color Table
